use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007004
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API
Alders
Alders'
CPAN
Container
DB
Data
Dave
Decoder
Eilam
Eilam's
EndMarker
GeoIP
GeoLite
GitHub
Greg
HasDecoder
HasMetadata
Hunter
IP
IPv
Inc
MAXMIND
MAXMIND's
Mateu
MaxMind
MaxMind's
NodeReader
OO
Olaf
Oschwald
Oschwald's
PP
PayPal
Ran
Reader
Role
Rolsky
Rolsky's
Stevenson
Sysreader
VERSIONING
William
YYY
YYYZZZ
bin
downloadable
drolsky
goschwald
lib
libmaxminddb
mhunter
mmdb
oalders
reilam
skyblue
versioning
